
sub newConfiguration {
    $Configuration = Configuration->new() ; 
    my $station = Station->new() ;
    $Configuration->station = $station ;

    $Configuration->datalogger = Datalogger->new() ;
    $Configuration->index = 1 ;

    my $SensorDir = pfget($Pf, "SensorDir" ) ; 
    if ( ! defined $SensorDir ) { 
	die ( "invalid or non-existent parameter file $Pf" ) ; 
    }
    if ( $SensorDir !~ m"^/" ) { 
	$SensorDir = "$ENV{ANTELOPE}/data/$SensorDir" ; 
    }
    $Configuration->SensorDir = $SensorDir ;

    my @suffixlist = qw(.pf) ; 
    my $ref = [] ;
    foreach my $choice ( <$SensorDir/*> ) { 
	my $basename = basename($choice,@suffixlist);
	push (@$ref, $basename) ;
    }
    $Configuration->possible_sensors=$ref ;

    my $DataloggerDir = pfget($Pf, "DataloggerDir" ) ; 
    if ( $DataloggerDir !~ m"^/" ) { 
	$DataloggerDir = "$ENV{ANTELOPE}/data/$DataloggerDir" ; 
    }
    $Configuration->DataloggerDir = $DataloggerDir ;

    $ref = [] ;
    foreach my $choice ( <$DataloggerDir/*> ) { 
	my $basename = basename($choice,@suffixlist);
	push (@$ref, $basename) ;
    }
    $Configuration->possible_dataloggers = $ref ;

    my $PreampDir = pfget($Pf, "PreampDir" ) ; 
    if ( $PreampDir !~ m"^/" ) { 
	$PreampDir = "$ENV{ANTELOPE}/data/$PreampDir" ; 
    }
    $Configuration->PreampDir = $PreampDir ;

    my $TemplateDir = pfget($Pf, "TemplateDir" ) ; 
    if ( $TemplateDir !~ m"^/" ) { 
	$TemplateDir = "$ENV{ANTELOPE}/data/$TemplateDir" ; 
    }
    $Configuration->TemplateDir = $TemplateDir ;

    $ref = [] ;
    foreach my $choice ( <$TemplateDir/*> ) { 
	my $basename = basename($choice);
	push (@$ref, $basename) ;
    }
    $Configuration->templates = $ref ;

    $Configuration->net = pfget($Pf, "default_network" ) ;
    my @db = dblookup (@Db, 0, "network", "net", $Configuration->net ) ;
    if ( $db[3] >= 0 ) {
	($Configuration->netname) = dbgetv ( @db, qw(netname)) ; 
    }
}

sub lookup_station { 
    lookup_sta() ;
    lookup_net() ;
}

sub lookup_sta { 
    my $station = $Configuration->station ;
    my $sta = $station->sta ; 
    my @db = dblookup (@Db, 0, "site", 0, 0) ; 
    $db[3] = dbSCRATCH ; 
    $offdate = nullvalue("offdate") ; 
    dbputv ( @db, "sta", $sta, "offdate", $offdate) ; 
    @records = dbmatches ( @db, @db, "ahook", "sta", "offdate" ) ;
    if ( @records > 0 ) { 
	$db[3] = shift(@records) ;
	my ($staname, $lat, $lon, $elev) =  
		dbgetv ( @db, qw(staname lat lon elev)) ; 
	$station->staname = $staname ;
	$station->lat = $lat ;
	$station->lon = $lon ;
	$station->elev = $elev ;
    }
}

sub select_datalogger {
    my ( $choice) = @_ ;
    my $datalogger = $Configuration->datalogger ;
    my $pf = $Configuration->DataloggerDir . "/$choice" ; 
    if ( ! -f "$pf.pf" ) {
	elog_complain ( "no definition for datalogger '$choice' in '$pf.pf'" ) ; 
	return 1 ; 
    }
    if ( validate_datalogger($pf) ) { 
	elog_complain ( "bad datalogger parameter file $pf" ) ; 
	return 1 ; 
    }
    $datalogger->dlchoice = $choice ;
    $datalogger->dlchosen = $choice ;
    $datalogger->pf = $pf ;
    $datalogger->dldesc = pfget ($pf, "description" ) ;
    $datalogger->dfile = pfget($pf, "dfile" ) ; 
    $datalogger->datalogger_serialnumber = pfget($pf, "default_datalogger_sn" ) ; 
    my $ref = pfget($pf, "streams") ; 
    my @streams = split(' ', $ref) ;
    $datalogger->possible_samprates = \@streams ; 
    &compile_pfroutine($pf, "possible_streams") ; 
    &compile_pfroutine($pf, "specify_filters") ; 
}

sub set_selected_sensor_by_name {
    my ($choice) = @_ ;
    my $result = 0 ;
    my $ref = $Configuration->sensors ; 
    foreach my $sensor ( @$ref ) { 
	if ( $sensor->snchoice eq $choice ) { 
	    $Configuration->selected_sensor= $sensor ; 
	    $result = 1 ;
	    last ;
	}
    }
    return $result ; 
}

sub batch { 
    my ($filename) = @_ ; 
    my $IN ; # = $Handle++ ; 
    open($IN, $filename) 
	or elog_complain ( "Can't open '$filename': $!" ) ;

    elog_log ( "\nloading $_[0]" ) ; 

    my ( $cmd, $sensor) ; 

    my $sensor_index = 0 ; 
    my @batch = () ;
    while ( <$IN> ) { 
	push (@batch, $_) ;
	chomp() ; 

	if ( -t STDOUT ) {
	    elog_log("-> $_") if $opt_v && $_ !~ /^\s*$/ ; 
	} else { 
	    print STDERR "-> $_\n" if $opt_v ; 
	}

	s/#.*// ; 
	s/^\s*// ;
	s/\s*$// ;
	next if /^$/ ; 

	if ( /^\s*(\S+)\s+(.*)/ ) { 
	    $cmd = $1 ; 
	    $_ = $2 ; 
	} else { 
	    $cmd = $_ ; 
	    $_ = "" ;
	}

	if ( $cmd eq "time" ) { 
	    my ($time, $msg) = is_epoch_string($_) ; 
	    if ( defined $time ) { 
		$Configuration->time =  strtime($time) ; 
	    } else { 
		elog_log ( "invalid time string '$_' : $msg" ) ; 
	    }
	} elsif ( $cmd eq "net" ) { 
	    my ($net, $netname) = split(' ', $_, 2) ; 
	    if ( ! defined $net ) { 
		elog_log ( "must specify network code" ) ; 
	    } elsif ( ! defined $netname ) { 
		elog_log ( "must specify network name" ) ; 
	    } 
	    $Configuration->net = $net ; 
	    $Configuration->netname = $netname ; 

	} elsif ( $cmd eq "sta" ) { 
	    my $station = $Configuration->station ;
	    my ($sta, $lat, $lon, $elev, $staname) = split(' ', $_, 5) ; 
	    if ( ! defined $sta ) { 
		elog_log ( "must specify station code" ) ; 
	    } else { 
		$station->sta = $sta ; 
		    
		if ( ! defined $lat ) {
		    if ( &lookup_station() != 0 ) { 
			elog_log ( "Can't find station $sta in site table" ) ;
		    }
		} elsif ( ! defined $lon ) { 
		    elog_log ( "must specify longitude" ) ; 
		} elsif ( ! defined $elev ) { 
		    elog_log ( "must specify elevation" ) ; 
		} elsif ( ! defined $staname ) { 
		    elog_log ( "must specify station name" ) ; 
		} else { 
		    $station->lat = $lat ; 
		    $station->lon = $lon ; 
		    $station->elev = $elev ; 
		    $station->staname = $staname ; 
		}
	    }

	} elsif ( $cmd eq "datalogger" ) { 
	    $Configuration->index = 1 ;
	    $Configuration->sensors = [] ; 
	    $Configuration->selected_sensor = undef ; 
	    my $datalogger = $Configuration->datalogger ;
	    my ($code, $sn, $dlsta) = split(' ', $_, 3) ; 
	    if ( ! defined $code ) { 
		elog_log ( "must specify datalogger code" ) ; 
	    } elsif ( ! defined $sn ) { 
		elog_log ( "must specify datalogger serial number" ) ; 
	    } else { 
		&select_datalogger($code) ;
		$datalogger->datalogger_serialnumber = $sn ;
		$datalogger->datalogger_id = $dlsta if specified($dlsta) ; 
	    }

	} elsif ( $cmd eq $Configuration->datalogger->dlchoice ) { 
	    my $datalogger = $Configuration->datalogger ;
	    my ($sn, $dlsta) = split(' ', $_, 2) ; 
	    $datalogger->datalogger_serialnumber = $sn if specified($sn) ; 
	    $datalogger->datalogger_id = $dlsta if specified($dlsta) ; 

	} elsif ( $cmd eq "sensor" ) { 
	    my ($code, $edepth, $sn, $loc) = split(' ', $_, 4) ; 
	    if ( ! defined $code ) { 
		elog_log ( "must specify sensor code" ) ; 
	    } elsif ( ! defined $edepth ) { 
		elog_log ( "must specify sensor emplacement depth" ) ; 
	    } elsif ( ! defined $sn ) { 
		elog_log ( "must specify sensor serial number" ) ; 
	    } else { 
		&select_sensor(undef, $code) ;
		$sensor = $Configuration->selected_sensor ;
		$sensor->edepth = $edepth ; 
		$sensor->sensor_serialnumber = $sn ; 
		$sensor->sensor_loc = $loc if specified($loc) ; 
	    }

	} elsif ( set_selected_sensor_by_name($cmd) ) { 
	    my ($edepth, $sn, $loc) = split(' ', $_, 3) ; 
	    $sensor = $Configuration->selected_sensor ;
	    $sensor->edepth = $edepth if specified($edepth) ; 
	    $sensor->sensor_serialnumber = $sn if specified($sn) ; 
	    $sensor->sensor_loc = $loc if specified($loc) ; 

	} elsif ( $cmd eq "axis" ) { 
	    my ($label, $hang, $vang, $gain, $lead, $preamp_gain, $preamp_stage)
		    = split(' ', $_, 7) ;
	    my $sensor = $Configuration->selected_sensor ;
	    if ( defined $sensor ) { 
		my $ref = $sensor->components() ;
		my @axes = @$ref ; 
		if ( ! defined $label ) { 
		    elog_log ( "must specify axis (component) label" ) ; 
		} elsif ( ! defined $hang || ! defined $vang ) { 
		    elog_log ( "must specify orientation angles hang and vang" ) ; 
		} else {
		    my $new_label ;
		    if ( $label =~ /^(.)->(.)$/ ) { 
			$label = $1 ; 
			$new_label = $2 ; 
		    } else { 
			$new_label = $label ; 
		    }
		    my $found = 0 ;
		    foreach my $axis ( @axes ) { 
			if ( $axis->axis eq $label ) { 
			    $axis->axis = $new_label ; 
			    $axis->by_hand = 1 ; 
			    $axis->hang = $hang if specified($hang) ;
			    $axis->vang = $vang if specified($vang) ;
			    $axis->sensor_gain = $gain if specified($gain) ;
			    $axis->lead = $lead if specified($lead) ;
			    $axis->preamp_gain = $preamp_gain if specified($preamp_gain) ;
			    $axis->preamp_stage = $preamp_stage if specified($preamp_stage) ;
			    rename_axis(undef, $axis) ;
			    $found = 1 ;
			    last ; 
			}
		    }
		    if ( ! $found ) { 
			elog_complain ( "Couldn't find axis '$axis' among defined axes: @axes" ) ; 
		    }
		}
	    } else { 
		elog_log ( "must specify sensor before axis" ) ; 
	    }

	} elsif ( $cmd eq "samplerate" ) {
	    my ($code, $cloc) = split(' ', $_) ;
	    my $sensor = $Configuration->selected_sensor ;
	    if ( defined $cloc ) { 
		my $save = $sensor->sensor_loc ;
		$sensor->sensor_loc = $cloc ; 
		$cloc = $save ; 
	    }
	    my $datalogger = $Configuration->datalogger ; 
	    if ( ! defined $datalogger ) { 
		elog_log ( "must specify datalogger before samplerate" ) ; 
	    } elsif ( ! defined $sensor ) { 
		elog_log ( "must specify sensor before samplerate" ) ; 
	    } else {
		@ok = possible_streams_now() ;
		if ( grep($code eq $_, @ok) ) { 
		    $sensor->selected_samplerate = select_samplerate( undef, $code ) ;
		} else { 
		    elog_log ("invalid samplerate $code : only @ok are allowed" ) ; 
		}
	    }
	    $sensor->sensor_loc = $cloc if defined $cloc ; 

	} elsif ( $cmd =~ /^(channel)$/ ) { 
	    my ($axis, $chan, $loc, $dlchan ) = split(' ', $_, 4) ;
	    $loc = '' if $loc eq "-" ; 
	    my $sensor = $Configuration->selected_sensor ;
	    if ( ! defined $sensor ) { 
		elog_log ( "must specify sensor before channel" ) ; 
	    } else {
		my $samplerate = $sensor->selected_samplerate ;
		if ( ! defined $samplerate ) {
		    elog_log ( "must specify samplerate before channel" ) ; 
		} else {
		    my $ref = $samplerate->channels ; 
		    my @channels = @$ref ;
		    my $found = 0 ; 
		    foreach my $channel ( @channels ) { 
			if ( $channel->component->axis eq $axis ) { 
			    if ( $chan eq "off" ) { 
				$channel->on = 0 ; 
			    } else { 
				$channel->on = 1 ; 
				$channel->chan = $chan ; 
				$channel->loc = $loc ; 
				$channel->id = $dlchan ;
			    }
			    $found = 1 ;
			    last ; 
			}
		    }
		    elog_log ( "no axis label '$axis' found" ) if ! $found ; 
		}
	    }

	} elsif ( $cmd eq "verify" ) { 
	    &run_dbverify() ;

	} elsif ( $cmd eq "show" ) { 
	    $Configuration->print() ;

	} elsif ( $cmd eq "dump" ) { 
	    $Configuration->dump() ;

	} elsif ( $cmd eq "template" ) { 
	    &batch($_) ;

	} elsif ( $cmd eq "add" ) { 
	    &save_to_database(@batch) ;
	    show_Added() ;
	    @batch = () ;
	    elog_log(" ") if $opt_v ;

	} else { 
	    elog_log ("unrecognized: '$cmd $_'")  ;
	}
    }
    close $IN ; 
}

sub add_sensor { 
    my $ref = $Configuration->sensors ;
    my $index = @$ref + 1 ; 
    my $sensor = Sensor->new(index=>$index) ;
    push(@$ref, $sensor) ;
    $Configuration->selected_sensor = $sensor ;
    return $sensor ;
}

sub select_sensor {
    my ($sensor, $choice) = @_ ;
    if (! defined $sensor) { 
	$sensor = add_sensor() ; 
    }
    $sensor->snchoice=$choice ;
    my $pf = $Configuration->SensorDir . "/$choice" ; 
    if ( ! -f "$pf.pf" ) {
	elog_complain ( "no definition for sensor '$choice' in '$pf.pf'" ) ; 
	return 1 ; 
    }
    if ( validate_sensor($pf) ) { 
	elog_complain ( "bad sensor parameter file $pf" ) ; 
	return 1 ; 
    }
    $sensor->pf = $pf ; 
    $sensor->sndesc = pfget($pf, "description") ; 
    $sensor->dfile = pfget($pf, "dfile" ) ; 
    $sensor->sensor_serialnumber = pfget($pf, "default_sensor_sn" ) ; 
    $sensor->rsptype = pfget($pf, "rsptype") ; 
    $sensor->band = pfget($pf, "band") ; 
    &setup_sensor_axes($sensor) ;
    return 0 ; 
}

sub setup_sensor_axes { 
    my ($sensor) = @_ ;
    my $pf = $sensor->pf ; 
    my $ref ;
    eval { 
	$ref = pfget($pf, "orientations") ; 
    } ; 
    if ( $@ ) { 
	elog_complain ("Can't get orientations from $pf" ) ; 
	return ; 
    }
    $sensor->sngnom = my $sngnom = pfget($pf, "gnom") + 0.0 ; 
    my @axes = @$ref ;
    set_orientations($sensor, @axes) ;
}

sub clear_samplerate {
    my ( $samplerate ) = @_ ; 
    my $channels = $samplerate->channels ; 
    foreach (@$channels) { 
	$_->chan = "" ; 
	$_->loc = "" ; 
	$_->id = "" ; 
    }
    $samplerate->samprate = "" ;
}

sub add_samplerate { 
    my $sensor = $Configuration->selected_sensor ; 
    my $samplerate = SampleRate->new() ; 
    my $components = $sensor->components ;
    my $names = $sensor->names ; 
    my $channels = $samplerate->channels ; 
    my $channel ;
    foreach my $component (@$components) { 
	$channel = Channel->new() ;
	$channel->component = $component ;
	push ( @$channels, $channel) ;
    }
    my $samplerates = $sensor->samplerates ; 
    push (@$samplerates, $samplerate) ; 
    return $samplerate ;
}

sub select_samplerate { 
    my ( $samplerate, $choice ) = @_ ; 
    if (! defined $samplerate) { 
	$samplerate = add_samplerate() ; 
    } else { 
	clear_samplerate($samplerate) ;
    }
    if ( $choice ne "" ) { 
	my $sensor = $Configuration->selected_sensor ;
	die ( "no sensor is selected!!" )  if ! defined $sensor ; 
	my $datalogger = $Configuration->datalogger ;
	my $pf = $datalogger->pf ; 
	my $stages = pfget ( $pf, $choice ) ; 
	if ( $stages eq "" ) { 
	    elog_complain ( "no samplerate '$choice' is defined for " . $sensor->snchoice ) ;
	    return undef ; 
	}
	$samplerate->choice = $choice ; 
	my @stages = split ( ' ', $stages ) ; 
	$samplerate->samprate = shift(@stages) ; 
	$stages = $samplerate->stages ; 
	push(@$stages, @stages) ;
	my ($name, $loc) = choose_channel_name ($samplerate, $sensor->sensor_loc) ;
	$samplerate->name = $name ; 
	my $channels = $samplerate->channels ; 
	foreach (@$channels) { 
	    $_->chan = $name . $_->component->axis ; 
	    $_->loc = $loc ;
	    $_->on = 1 ; 
	}
    }
    &validate_stream_stages() ; 
    return $samplerate ; 
}

1;

__END__ 
